@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: boardlist_ui.c,v 1.43 2003/12/21 19:14:02 fuyu Exp $
+ * $Id: boardlist_ui.c,v 1.45 2003/12/24 03:41:41 fuyu Exp $
  */
 
 #include "config.h"
@@ -60,6 +60,8 @@
 #include <string.h>
 #include <unistd.h>
 
+#include <zlib.h>
+
 
 #if DEBUG_WIDGET_MOST
 static void select_category_cb(BoardlistView *view,
@@ -97,11 +99,11 @@ static void boardlist_read_board_element_cb(OchushaBBSTable *table,
 
 static void boardlist_write_category_element_cb(OchushaBBSTable *table,
 						OchushaBoardCategory *category,
-						FILE *boardlist_xml,
+						gzFile *boardlist_xml,
 						OchushaApplication *appl);
 static void boardlist_write_board_element_cb(OchushaBBSTable *table,
 					     OchushaBulletinBoard *board,
-					     FILE *boardlist_xml,
+					     gzFile *boardlist_xml,
 					     OchushaApplication *appl);
 
 static void write_threadlist(gpointer data, gpointer user_data);
@@ -394,7 +396,8 @@ initialize_boardlist(OchushaApplication *application)
 		   application);
 
   if (ochusha_bbs_table_read_boardlist_xml(application->table,
-					   &application->config))
+					   &application->config,
+					   application->session_subdir))
     {
       setup_boardlist(application);
       return;
@@ -464,6 +467,9 @@ refresh_boardlist(OchushaApplication *application)
 void
 finalize_boardlist(OchushaApplication *application)
 {
+#if 0
+  fprintf(stderr, "finalize_boardlist\n");
+#endif
   if (application->contents_window != NULL)
     {
       PanedNotebook *boardlist = PANED_NOTEBOOK(application->contents_window);
@@ -480,8 +486,15 @@ finalize_boardlist(OchushaApplication *application)
   if (application->table != NULL)
     {
       if (!ochusha_bbs_table_write_boardlist_xml(application->table,
-						 &application->config))
-	fprintf(stderr, "Couldn't write boardlist2.xml.\n");
+						 &application->config,
+						 application->session_subdir))
+	fprintf(stderr, "Couldn't write boardlist.xml.\n");
+
+      if (application->session_subdir != NULL
+	  && !ochusha_bbs_table_write_boardlist_xml(application->table,
+						    &application->config,
+						    NULL))
+	fprintf(stderr, "Couldn't write boardlist.xml.\n");
     }
 
   g_slist_foreach(board_list, write_threadlist, application);
@@ -1170,12 +1183,12 @@ boardlist_read_board_element_cb(OchushaBBSTable *table,
 }
 
 
-#define OUTPUT_CATEGORY_ATTRIBUTE_BOOLEAN(file, category, attribute)	\
+#define OUTPUT_CATEGORY_ATTRIBUTE_BOOLEAN(gzfile, category, attribute)	\
   do									\
     {									\
       if ((category)->attribute)					\
 	{								\
-	  fprintf(file,							\
+	  gzprintf(gzfile,						\
 		  "      <attribute name=\"" #attribute	"\">\n"		\
 		  "        <boolean val=\"%s\"/>\n"			\
 		  "      </attribute>\n",				\
@@ -1187,7 +1200,7 @@ boardlist_read_board_element_cb(OchushaBBSTable *table,
 static void
 boardlist_write_category_element_cb(OchushaBBSTable *table,
 				    OchushaBoardCategory *category,
-				    FILE *boardlist_xml,
+				    gzFile *boardlist_xml,
 				    OchushaApplication *application)
 {
   BoardCategoryGUIInfo *info = ensure_board_category_info(category);
@@ -1196,12 +1209,12 @@ boardlist_write_category_element_cb(OchushaBBSTable *table,
 }
 
 
-#define OUTPUT_BOARD_ATTRIBUTE_BOOLEAN(file, board, attribute)		\
+#define OUTPUT_BOARD_ATTRIBUTE_BOOLEAN(gzfile, board, attribute)	\
   do									\
     {									\
       if ((board)->attribute)						\
 	{								\
-	  fprintf(file,							\
+	  gzprintf(gzfile,						\
 		  "        <attribute name=\"" #attribute "\">\n"	\
 		  "          <boolean val=\"%s\"/>\n"			\
 		  "        </attribute>\n",				\
@@ -1210,12 +1223,12 @@ boardlist_write_category_element_cb(OchushaBBSTable *table,
     } while (0)
 
 
-#define OUTPUT_BOARD_ATTRIBUTE_INT(file, board, attribute)		\
+#define OUTPUT_BOARD_ATTRIBUTE_INT(gzfile, board, attribute)		\
   do									\
     {									\
       if ((board)->attribute)						\
 	{								\
-	  fprintf(file,							\
+	  gzprintf(gzfile,						\
 		  "        <attribute name=\"" #attribute "\">\n"	\
 		  "          <int val=\"%d\"/>\n"			\
 		  "        </attribute>\n", (board)->attribute);	\
@@ -1223,12 +1236,12 @@ boardlist_write_category_element_cb(OchushaBBSTable *table,
     } while (0)
 
 
-#define OUTPUT_BOARD_ATTRIBUTE_STRING(file, board, attribute)		\
+#define OUTPUT_BOARD_ATTRIBUTE_STRING(gzfile, board, attribute)		\
   do {									\
     if ((board)->attribute != NULL)					\
       {									\
 	gchar *text = g_markup_escape_text((board)->attribute, -1);	\
-	fprintf(file,							\
+	gzprintf(gzfile,						\
 		"        <attribute name=\"" #attribute "\">\n"		\
 		"          <string>%s</string>\n"			\
 		"        </attribute>\n", text);			\
@@ -1240,7 +1253,7 @@ boardlist_write_category_element_cb(OchushaBBSTable *table,
 static void
 boardlist_write_board_element_cb(OchushaBBSTable *table,
 				 OchushaBulletinBoard *board,
-				 FILE *boardlist_xml,
+				 gzFile *boardlist_xml,
 				 OchushaApplication *application)
 {
   BulletinBoardGUIInfo *info = ensure_bulletin_board_info(board, application);
@@ -1280,8 +1293,18 @@ write_threadlist(gpointer data, gpointer user_data)
   OchushaBulletinBoard *board = (OchushaBulletinBoard *)data;
   OchushaApplication *application = (OchushaApplication *)user_data;
 
+#if 0
+  fprintf(stderr, "write_threadlist: base_url=%s\n", board->base_url);
+#endif
   /* XXX: 顼 */
-  if (!ochusha_bulletin_board_write_threadlist_xml(board, &application->config))
+  if (!ochusha_bulletin_board_write_threadlist_xml(board, &application->config,
+						application->session_subdir))
+    fprintf(stderr, "Couldn't write threadlist2.xml.\n");
+
+  if (application->session_subdir != NULL
+      && !ochusha_bulletin_board_write_threadlist_xml(board,
+						      &application->config,
+						      NULL))
     fprintf(stderr, "Couldn't write threadlist2.xml.\n");
 }
 
@@ -1453,7 +1476,11 @@ build_boardlist(WorkerThread *employee, gpointer args)
       setup_boardlist(application);
       gdk_threads_leave();
 
-      ochusha_bbs_table_write_boardlist_xml(table, &application->config);
+      ochusha_bbs_table_write_boardlist_xml(table, &application->config,
+					    application->session_subdir);
+      if (application->session_subdir != NULL)
+	ochusha_bbs_table_write_boardlist_xml(table, &application->config,
+					      NULL);
     }
 
  cleanup_before_return:
