@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: bulletin_board_ui.c,v 1.80 2003/12/21 02:23:13 fuyu Exp $
+ * $Id: bulletin_board_ui.c,v 1.81 2003/12/24 03:41:41 fuyu Exp $
  */
 
 #include "config.h"
@@ -59,6 +59,8 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include <zlib.h>
+
 
 static void open_thread_view(WorkerThread *employee, gpointer job_args);
 static void open_kako_thread_view(WorkerThread *employee, gpointer job_args);
@@ -303,12 +305,12 @@ threadlist_read_thread_element_cb(OchushaBulletinBoard *board,
 }
 
 
-#define OUTPUT_THREAD_ATTRIBUTE_BOOLEAN(file, thread, attribute)	\
+#define OUTPUT_THREAD_ATTRIBUTE_BOOLEAN(gzfile, thread, attribute)	\
   do									\
     {									\
       if ((thread)->attribute)						\
 	{								\
-	  fprintf(file,							\
+	  gzprintf(gzfile,						\
 		  "      <attribute name=\"" #attribute	"\">\n"		\
 		  "        <boolean val=\"%s\"/>\n"			\
 		  "      </attribute>\n",				\
@@ -317,12 +319,12 @@ threadlist_read_thread_element_cb(OchushaBulletinBoard *board,
     } while (0)
 
 
-#define OUTPUT_THREAD_ATTRIBUTE_INT(file, thread, attribute)		\
+#define OUTPUT_THREAD_ATTRIBUTE_INT(gzfile, thread, attribute)		\
   do									\
     {									\
       if ((thread)->attribute)						\
 	{								\
-	  fprintf(file,							\
+	  gzprintf(gzfile,						\
 		  "      <attribute name=\"" #attribute	"\">\n"		\
 		  "        <int val=\"%d\"/>\n"				\
 		  "      </attribute>\n", (thread)->attribute);		\
@@ -330,12 +332,12 @@ threadlist_read_thread_element_cb(OchushaBulletinBoard *board,
     } while (0)
 
 
-#define OUTPUT_THREAD_ATTRIBUTE_STRING(file, thread, attribute)		\
+#define OUTPUT_THREAD_ATTRIBUTE_STRING(gzfile, thread, attribute)	\
   do {									\
     if ((thread)->attribute != NULL)					\
       {									\
 	gchar *text = g_markup_escape_text((thread)->attribute, -1);	\
-	fprintf(file,							\
+	gzprintf(gzfile,						\
 		"      <attribute name=\"" #attribute	"\">\n"		\
 		"        <string>%s</string>\n"				\
 		"      </attribute>\n", text);				\
@@ -348,7 +350,7 @@ threadlist_read_thread_element_cb(OchushaBulletinBoard *board,
 static void
 threadlist_write_thread_element_cb(OchushaBulletinBoard *board,
 				   OchushaBBSThread *thread,
-				   FILE *threadlist_xml,
+				   gzFile *threadlist_xml,
 				   OchushaApplication *application)
 {
   BBSThreadGUIInfo *info = ensure_bbs_thread_info(thread);
@@ -1308,6 +1310,7 @@ show_threadlist(WorkerThread *employee, gpointer args)
 
   /* ˽λʤäˤ٤ */
   ochusha_bulletin_board_read_threadlist_xml(board, &application->config,
+					     application->session_subdir,
 					     info->recover_mode);
   info->recover_mode = FALSE;
 
