@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_ui.c,v 1.53 2003/12/21 04:36:08 fuyu Exp $
+ * $Id: ochusha_ui.c,v 1.55 2003/12/24 03:41:41 fuyu Exp $
  */
 
 #include "config.h"
@@ -43,7 +43,8 @@
 #include "icon_label.h"
 #include "paned_notebook.h"
 #include "regex_utils.h"
-
+#include "session.h"
+#include "xsmp.h"
 #include "ugly_gtk2utils.h"
 
 #include <glib.h>
@@ -52,12 +53,17 @@
 
 #include <pthread.h>
 
+#ifdef HAVE_SIGNAL_H
+#include <signal.h>
+#endif
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
 #include <time.h>
 
+#include <zlib.h>
+
 #include "write_response_24.h"
 
 #include "blue0.xpm"
@@ -72,7 +78,10 @@
 
 #include "ochusha-banner.h"
 #include "ochusha-banner1.h"
+#define XMAS_SPECIAL	1
+#ifdef XMAS_SPECIAL
 #include "ochusha-banner2.h"
+#endif
 
 
 static void setup_ochusha_stock_items(void);
@@ -91,7 +100,7 @@ static void cancel_button_cb(gpointer unused, OchushaApplication *application);
 
 static void on_quit(gpointer data, guint action, GtkWidget *widget);
 static void main_menu_callback(gpointer data, guint action, GtkWidget *widget);
-static void write_optional_prefs(FILE *file, gpointer user_data);
+static void write_optional_prefs(gzFile *file, gpointer user_data);
 static void configure_application(OchushaApplication *application);
 static void read_optional_prefs(GHashTable *pref_attrs, gpointer user_data);
 
@@ -158,7 +167,9 @@ static GtkWidget *preference_dialog = NULL;
 static GtkWidget *about_dialog = NULL;
 static GdkPixbuf *banner_pixbuf = NULL;
 static GdkPixbuf *banner1_pixbuf = NULL;
+#ifdef XMAS_SPECIAL
 static GdkPixbuf *banner2_pixbuf = NULL;
+#endif
 static GtkWidget *open_url_dialog = NULL;
 static GtkWidget *open_url_combo = NULL;
 static GtkWidget *open_url_in_tab_button = NULL;
@@ -171,6 +182,33 @@ static GtkTextIter log_tail_iter;
 #endif
 
 
+
+
+#ifdef HAVE_SIGNAL_H
+static OchushaApplication *application_global = NULL;
+
+static RETSIGTYPE
+terminate_on_signal(int sig)
+{
+  OchushaApplication *application = application_global;
+#if 0
+  fprintf(application->log_file, "terminate_on_signal: sig=%d\n", sig);
+  fflush(application->log_file);
+#endif
+  if (application == NULL)
+    {
+      fprintf(stderr, "Ochusha terminated on unexpected signal.\n");
+      exit(0);
+    }
+
+  if (!application->states_saved)
+    ochusha_session_save_states(application);
+
+  gtk_main_quit();
+}
+#endif
+
+
 void
 ochusha_init_application(OchushaApplication *application)
 {
@@ -330,6 +368,9 @@ ochusha_init_application(OchushaApplication *application)
       abort();
     }
 
+  if (application->sm_client_id != NULL)
+    gdk_set_sm_client_id(application->sm_client_id);
+
   setup_ochusha_stock_items();
   setup_tab_label_icons();
 
@@ -337,8 +378,10 @@ ochusha_init_application(OchushaApplication *application)
     = gdk_pixbuf_new_from_inline(-1, ochusha_banner_inline, FALSE, NULL);
   banner1_pixbuf
     = gdk_pixbuf_new_from_inline(-1, ochusha_banner1_inline, FALSE, NULL);
+#ifdef XMAS_SPECIAL
   banner2_pixbuf
     = gdk_pixbuf_new_from_inline(-1, ochusha_banner2_inline, FALSE, NULL);
+#endif
 
   configure_application(application);
 
@@ -354,6 +397,15 @@ ochusha_init_application(OchushaApplication *application)
 
   gtk_window_set_icon(application->top_level,
 		      gdk_pixbuf_new_from_inline(-1, ochusha48_inline, FALSE, NULL));
+  gtk_widget_set_size_request(GTK_WIDGET(application->top_level),
+			      application->width, application->height);
+  gtk_widget_realize(GTK_WIDGET(application->top_level));
+  if (application->x_pos >= 0 && application->y_pos >= 0)
+    {
+      gtk_window_move(application->top_level,
+		      application->x_pos, application->y_pos);
+      gtk_widget_queue_resize(GTK_WIDGET(application->top_level));
+    }
 
   application->main_window = GTK_BOX(gtk_vbox_new(FALSE, 0));
   gtk_container_add(GTK_CONTAINER(application->top_level),
@@ -363,12 +415,9 @@ ochusha_init_application(OchushaApplication *application)
 		   G_CALLBACK(gtk_widget_destroyed),
 		   &application->main_window);
 #endif
-
   gtk_window_set_geometry_hints(application->top_level,
 				GTK_WIDGET(application->main_window),
 				&geometry, GDK_HINT_MIN_SIZE);
-  gtk_widget_set_size_request(GTK_WIDGET(application->top_level),
-			      application->width, application->height);
 
   /* menu barŹ */
   accel_group = gtk_accel_group_new();
@@ -498,6 +547,15 @@ ochusha_init_application(OchushaApplication *application)
 #endif
 
   gtk_widget_show_all(GTK_WIDGET(application->top_level));
+
+#ifdef HAVE_SIGNAL_H
+  application_global = application;
+  if (signal(SIGTERM, terminate_on_signal) == SIG_ERR)
+    {
+      fprintf(stderr,
+	      "sigaction failed: ochusha will continue to work anyway.\n");
+    }
+#endif
 }
 
 
@@ -872,26 +930,30 @@ cancel_button_cb(gpointer unused, OchushaApplication *application)
 static void
 on_quit(gpointer data, guint action, GtkWidget *widget)
 {
-  OchushaApplication *application = (OchushaApplication *)data;
-  GtkAllocation *allocation = &GTK_WIDGET(application->top_level)->allocation;
+  ochusha_session_save_states((OchushaApplication *)data);
 
-  application->width = allocation->width;
-  application->height = allocation->height;
-
-  allocation = &application->boardlist_view->allocation;
-  application->boardlist_width = allocation->width;
-
-  finalize_boardlist(application);
-
-  application->shutdown_succeeded = TRUE;
-  if (!ochusha_write_config_xml(&application->config,
-				write_optional_prefs, application))
-    fprintf(stderr, "Couldn't write %s.\n", OCHUSHA_CONFIG_XML);
+  finalize_xsmp_handlers((OchushaApplication *)data);
 
   gtk_main_quit();
 }
 
 
+void
+ochusha_write_preferences(OchushaApplication *application)
+{
+  if (!ochusha_write_config_xml(&application->config,
+				application->session_subdir,
+				write_optional_prefs, application))
+    fprintf(stderr, "Couldn't write %s/%s.\n",
+	    application->session_subdir, OCHUSHA_CONFIG_XML);
+
+  if (application->session_subdir != NULL
+      && !ochusha_write_config_xml(&application->config, NULL,
+				   write_optional_prefs, application))
+    fprintf(stderr, "Couldn't write %s.\n", OCHUSHA_CONFIG_XML);
+}
+
+
 static void
 main_menu_callback(gpointer data, guint action, GtkWidget *widget)
 {
@@ -964,24 +1026,24 @@ main_menu_callback(gpointer data, guint action, GtkWidget *widget)
 }
 
 
-#define OUTPUT_CONFIG_ATTRIBUTE_INT(file, config, attribute)	\
-  fprintf(file,							\
+#define OUTPUT_CONFIG_ATTRIBUTE_INT(gzfile, config, attribute)	\
+  gzprintf(gzfile,						\
 	  "    <attribute name=\"" #attribute	"\">\n"		\
 	  "      <int val=\"%d\"/>\n"				\
 	  "    </attribute>\n", (config)->attribute)
 
-#define OUTPUT_CONFIG_ATTRIBUTE_BOOLEAN(file, config, attribute)	\
-  fprintf(file,								\
+#define OUTPUT_CONFIG_ATTRIBUTE_BOOLEAN(gzfile, config, attribute)	\
+  gzprintf(gzfile,							\
 	  "    <attribute name=\"" #attribute	"\">\n"			\
 	  "      <boolean val=\"%s\"/>\n"				\
 	  "    </attribute>\n", (config)->attribute ? "true" : "false" )
 
-#define OUTPUT_CONFIG_ATTRIBUTE_STRING(file, config, attribute)		\
+#define OUTPUT_CONFIG_ATTRIBUTE_STRING(gzfile, config, attribute)	\
   do {									\
     if ((config)->attribute != NULL)					\
       {									\
 	gchar *text = g_markup_escape_text((config)->attribute, -1);	\
-	fprintf(file,							\
+	gzprintf(gzfile,						\
 		"    <attribute name=\"" #attribute	"\">\n"		\
 		"      <string>%s</string>\n"				\
 		"    </attribute>\n", text);				\
@@ -991,7 +1053,7 @@ main_menu_callback(gpointer data, guint action, GtkWidget *widget)
 
 
 static void
-write_optional_prefs(FILE *file, gpointer user_data)
+write_optional_prefs(gzFile *file, gpointer user_data)
 {
   OchushaApplication *application = (OchushaApplication *)user_data;
   gchar *bookmark_text;
@@ -1036,7 +1098,7 @@ write_optional_prefs(FILE *file, gpointer user_data)
       if (bookmark_text != NULL && *bookmark_text != '\0')
 	{
 	  gchar *text = g_markup_escape_text(bookmark_text, -1);
-	  fprintf(file,
+	  gzprintf(file,
 		  "    <attribute name=\"bookmark_text\">\n"
 		  "      <string>%s</string>\n"
 		  "    </attribute>\n", text);
@@ -1097,8 +1159,9 @@ static void
 configure_application(OchushaApplication *application)
 {
   gboolean succeeded;
-  ochusha_read_config_xml(&application->config, read_optional_prefs,
-			  application);
+
+  ochusha_read_config_xml(&application->config, application->session_subdir,
+			  read_optional_prefs, application);
   application->broker = ochusha_network_broker_new(&application->config);
   g_signal_connect(G_OBJECT(application->broker), "access_started",
 		   G_CALLBACK(access_started_cb), application);
@@ -1116,6 +1179,8 @@ configure_application(OchushaApplication *application)
   if (application->width == 0)
     {	/* ƵưȻפ */
       /* ɥν */
+      application->x_pos = -1;
+      application->y_pos = -1;
       application->width = OCHUSHA_DEFAULT_WIDTH;
       application->height = OCHUSHA_DEFAULT_HEIGHT;
       application->boardlist_width = OCHUSHA_DEFAULT_BOARDLIST_WIDTH;
@@ -1205,14 +1270,17 @@ configure_application(OchushaApplication *application)
 
   succeeded = application->shutdown_succeeded;
   application->shutdown_succeeded = FALSE;
-  if (!ochusha_write_config_xml(&application->config,
-				write_optional_prefs, application))
-    fprintf(stderr, "Couldn't write %s.\n", OCHUSHA_CONFIG_XML);
+  ochusha_write_preferences(application);
   application->shutdown_succeeded = succeeded;
+
   if (!succeeded)
     application->recover_mode = TRUE;
   else
     application->recover_mode = FALSE;
+
+  application->ice_connection_poll_id = 0;
+
+  ochusha_session_restore_states(application);
 }
 
 
@@ -2603,10 +2671,12 @@ open_about_dialog(OchushaApplication *application)
 	  time_t utc = time(NULL);
 	  localtime_r(&utc, &tm);
 
-	  if (tm.tm_yday >= 357 && tm.tm_yday <= 358)
-	    banner = banner2_pixbuf;
-	  else if (tm.tm_year == 104 && tm.tm_yday < 7)
+	  if (tm.tm_year == 104 && tm.tm_yday < 7)
 	    banner = banner1_pixbuf;
+#ifdef XMAS_SPECIAL
+	  else if (tm.tm_yday >= 357 && tm.tm_yday <= 358)
+	    banner = banner2_pixbuf;
+#endif
 	  else
 	    banner = banner_pixbuf;
 	}
