@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_ui.h,v 1.40 2003/12/20 04:38:44 fuyu Exp $
+ * $Id: ochusha_ui.h,v 1.43 2003/12/24 05:17:03 fuyu Exp $
  */
 
 #ifndef _OCHUSHA_UI_H_
@@ -52,6 +52,8 @@
 #define ENABLE_DEBUG_CONSOLE						0
 #define ENABLE_MAIN_TOOLBAR						0
 
+#define OCHUSHA_SESSION_XML				"session.xml.gz"
+
 #define OCHUSHA_DEFAULT_WIDTH						600
 #define OCHUSHA_DEFAULT_HEIGHT						400
 #define OCHUSHA_DEFAULT_BOARDLIST_WIDTH					150
@@ -167,7 +169,9 @@ struct _OchushaApplication
   OchushaBBSTable *table;
 
 
-  /* windowʤ */
+  /* window(ǥե)ʤ */
+  gint x_pos;
+  gint y_pos;
   gint width;
   gint height;
   gint boardlist_width;
@@ -250,6 +254,24 @@ struct _OchushaApplication
   /* ˽λɤ */
   gboolean shutdown_succeeded;
   gboolean recover_mode;
+  gboolean states_saved;
+
+
+  /* XSMPϢ */
+  int argc;
+  char **argv;
+
+  gpointer ice_connection;
+  guint ice_connection_poll_id;
+
+  gpointer smc_connection;
+  char *sm_client_id;
+  char *previous_sm_client_id;
+
+  char *session_id;
+  char *session_subdir;
+
+  FILE *log_file;
 
   OchushaConfig config;
 };
@@ -272,6 +294,7 @@ typedef enum
 } OchushaMainMenuAction;
 
 
+void ochusha_write_preferences(OchushaApplication *application);
 void ochusha_init_application(OchushaApplication *application);
 void ochusha_set_main_menu_sensitive(OchushaMainMenuAction action,
 				     gboolean sensitive);
