@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: paned_notebook.c,v 1.8 2003/12/05 08:41:26 fuyu Exp $
+ * $Id: paned_notebook.c,v 1.9 2003/12/21 02:23:13 fuyu Exp $
  */
 
 #include "config.h"
@@ -457,7 +457,10 @@ paned_notebook_finalize(GObject *object)
 {
 #if 0
   PanedNotebook *paned_notebook = (PanedNotebook *)object;
+
+  fprintf(stderr, "paned_notebook_finalize(%p)\n", paned_notebook);
 #endif
+
   if (G_OBJECT_CLASS(parent_class)->finalize)
     (*G_OBJECT_CLASS(parent_class)->finalize)(object);
 }
@@ -496,6 +499,10 @@ paned_notebook_destroy(GtkObject *object)
 {
   PanedNotebook *paned_notebook = (PanedNotebook *)object;
 
+#if 0
+  fprintf(stderr, "paned_notebook_destroy(%p)\n", paned_notebook);
+#endif
+
   if (paned_notebook->items_shown != NULL)
     {
       g_slist_foreach(paned_notebook->items_shown,
@@ -1264,7 +1271,7 @@ close_nth_page(PanedNotebook *paned_notebook, gint page)
   g_return_if_fail(widget != NULL);
 
   relation = find_item_view_relation(paned_notebook, widget);
-#if DEBUG_WIDGET
+#if 1 || DEBUG_WIDGET
   g_return_if_fail(relation != NULL && relation->item_view == widget);
 #else
   if (relation == NULL || relation->item_view != widget)
