@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_bbs_table.c,v 1.17 2003/12/19 03:13:49 fuyu Exp $
+ * $Id: ochusha_bbs_table.c,v 1.18 2003/12/24 03:40:50 fuyu Exp $
  */
 
 #include "config.h"
@@ -49,6 +49,8 @@
 #include <string.h>
 #include <unistd.h>
 
+#include <zlib.h>
+
 
 static void ochusha_bbs_table_class_init(OchushaBBSTableClass *klass);
 static void ochusha_bbs_table_init(OchushaBBSTable *table);
@@ -934,7 +936,8 @@ getEntityHandler(void *context, const xmlChar *name)
 
 gboolean
 ochusha_bbs_table_read_boardlist_xml(OchushaBBSTable *table,
-				     OchushaConfig *config)
+				     OchushaConfig *config,
+				     const char *subdir)
 {
   SAXContext context =
     {
@@ -949,7 +952,7 @@ ochusha_bbs_table_read_boardlist_xml(OchushaBBSTable *table,
   char *pathname;
   g_return_val_if_fail(OCHUSHA_IS_BBS_TABLE(table) && config != NULL, FALSE);
 
-  pathname = ochusha_config_find_file(config, OCHUSHA_BOARDLIST_XML);
+  pathname = ochusha_config_find_file(config, OCHUSHA_BOARDLIST_XML, subdir);
   if (pathname == NULL)
     return FALSE;
 
@@ -989,7 +992,7 @@ ochusha_bbs_table_read_boardlist_xml(OchushaBBSTable *table,
 typedef struct _WriteBoardlistArgs
 {
   OchushaBBSTable *table;
-  FILE *boardlist_xml;
+  gzFile *boardlist_xml;
 } WriteBoardlistArgs;
 
 
@@ -998,12 +1001,12 @@ write_board(gpointer data, gpointer user_data)
 {
   OchushaBulletinBoard *board = OCHUSHA_BULLETIN_BOARD(data);
   WriteBoardlistArgs *args = (WriteBoardlistArgs *)user_data;
-  FILE *boardlist_xml = args->boardlist_xml;
+  gzFile *boardlist_xml = args->boardlist_xml;
 
   if (board->killed)
     return;
 
-  fprintf(boardlist_xml, "      <board>\n");
+  gzprintf(boardlist_xml, "      <board>\n");
 
   g_signal_emit_by_name(G_OBJECT(board),
 			"write_boardlist_element",
@@ -1018,7 +1021,7 @@ write_board(gpointer data, gpointer user_data)
 		board,
 		boardlist_xml);
 
-  fprintf(boardlist_xml, "      </board>\n");
+  gzprintf(boardlist_xml, "      </board>\n");
 }
 
 
@@ -1027,12 +1030,12 @@ write_category(gpointer data, gpointer user_data)
 {
   OchushaBoardCategory *category = OCHUSHA_BOARD_CATEGORY(data);
   WriteBoardlistArgs *args = (WriteBoardlistArgs *)user_data;
-  FILE *boardlist_xml = args->boardlist_xml;
+  gzFile *boardlist_xml = args->boardlist_xml;
 
   if (category->killed)
     return;
 
-  fprintf(boardlist_xml, "    <category>\n");
+  gzprintf(boardlist_xml, "    <category>\n");
 
   g_signal_emit_by_name(G_OBJECT(category),
 			"write_boardlist_element",
@@ -1045,22 +1048,23 @@ write_category(gpointer data, gpointer user_data)
 
   g_slist_foreach(category->board_list, write_board, args);
 
-  fprintf(boardlist_xml, "    </category>\n");
+  gzprintf(boardlist_xml, "    </category>\n");
 }
 
 
 gboolean
 ochusha_bbs_table_write_boardlist_xml(OchushaBBSTable *table,
-				      OchushaConfig *config)
+				      OchushaConfig *config,
+				      const char *subdir)
 {
   WriteBoardlistArgs args;
-  FILE *boardlist_xml;
+  gzFile *boardlist_xml;
   int fd;
 
   g_return_val_if_fail(OCHUSHA_IS_BBS_TABLE(table) && config != NULL, FALSE);
   g_return_val_if_fail(config->home != NULL, FALSE);
 
-  fd = ochusha_config_open_file(config, OCHUSHA_BOARDLIST_XML,
+  fd = ochusha_config_open_file(config, OCHUSHA_BOARDLIST_XML, subdir,
 				O_WRONLY | O_TRUNC | O_CREAT);
   if (fd < 0)
     {
@@ -1069,7 +1073,7 @@ ochusha_bbs_table_write_boardlist_xml(OchushaBBSTable *table,
       return FALSE;
     }
 
-  boardlist_xml = fdopen(fd, "w");
+  boardlist_xml = gzdopen(fd, "w");
   if (boardlist_xml == NULL)
     {
       close(fd);
@@ -1077,17 +1081,17 @@ ochusha_bbs_table_write_boardlist_xml(OchushaBBSTable *table,
       return FALSE;
     }
 
-  fprintf(boardlist_xml, "<?xml version=\"1.0\"?>\n");
-  fprintf(boardlist_xml, "<ochusha>\n");
-  fprintf(boardlist_xml, "  <boardlist>\n");
+  gzprintf(boardlist_xml, "<?xml version=\"1.0\"?>\n");
+  gzprintf(boardlist_xml, "<ochusha>\n");
+  gzprintf(boardlist_xml, "  <boardlist>\n");
 
   args.table = table;
   args.boardlist_xml = boardlist_xml;
 
   g_slist_foreach(table->category_list, write_category, &args);
 
-  fprintf(boardlist_xml, "  </boardlist>\n");
-  fprintf(boardlist_xml, "</ochusha>\n");
+  gzprintf(boardlist_xml, "  </boardlist>\n");
+  gzprintf(boardlist_xml, "</ochusha>\n");
 
-  return fclose(boardlist_xml) == 0;
+  return gzclose(boardlist_xml) == 0;
 }
