@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_bbs_thread.c,v 1.18 2003/12/19 03:13:49 fuyu Exp $
+ * $Id: ochusha_bbs_thread.c,v 1.19 2003/12/24 03:40:50 fuyu Exp $
  */
 
 #include "config.h"
@@ -42,6 +42,8 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include <zlib.h>
+
 
 static void ochusha_bbs_thread_class_init(OchushaBBSThreadClass *klass);
 static void ochusha_bbs_thread_init(OchushaBBSThread *thread);
@@ -58,7 +60,7 @@ static void ochusha_bbs_thread_read_threadlist_element(
 						GHashTable *thread_attributes);
 static void ochusha_bbs_thread_write_threadlist_element(
 						OchushaBBSThread *thread,
-						FILE *threadlist_xml);
+						gzFile *threadlist_xml);
 
 GType
 ochusha_bbs_thread_get_type(void)
@@ -603,12 +605,12 @@ ochusha_bbs_thread_read_threadlist_element(OchushaBBSThread *thread,
 /*
  * threadlist.xmlϢ
  */
-#define OUTPUT_THREAD_ATTRIBUTE_INT(file, thread, attribute)		\
+#define OUTPUT_THREAD_ATTRIBUTE_INT(gzfile, thread, attribute)		\
   do									\
     {									\
       if ((thread)->attribute)						\
 	{								\
-	  fprintf(file,							\
+	  gzprintf(gzfile,						\
 		  "      <attribute name=\"" #attribute	"\">\n"		\
 		  "        <int val=\"%d\"/>\n"				\
 		  "      </attribute>\n", (thread)->attribute);		\
@@ -616,12 +618,12 @@ ochusha_bbs_thread_read_threadlist_element(OchushaBBSThread *thread,
     } while (0)
 
 
-#define OUTPUT_THREAD_ATTRIBUTE_BOOLEAN(file, thread, attribute)	\
+#define OUTPUT_THREAD_ATTRIBUTE_BOOLEAN(gzfile, thread, attribute)	\
   do									\
     {									\
       if ((thread)->attribute)						\
 	{								\
-	  fprintf(file,							\
+	  gzprintf(gzfile,						\
 		  "      <attribute name=\"" #attribute	"\">\n"		\
 		  "        <boolean val=\"%s\"/>\n"			\
 		  "      </attribute>\n",				\
@@ -630,13 +632,13 @@ ochusha_bbs_thread_read_threadlist_element(OchushaBBSThread *thread,
     } while (0)
 
 
-#define OUTPUT_THREAD_ATTRIBUTE_STRING(file, thread, attribute)		\
+#define OUTPUT_THREAD_ATTRIBUTE_STRING(gzfile, thread, attribute)	\
   do									\
     {									\
       if ((thread)->attribute)						\
 	{								\
 	  gchar *text = g_markup_escape_text((thread)->attribute, -1);	\
-	  fprintf(file,							\
+	  gzprintf(gzfile,						\
 		  "      <attribute name=\"" #attribute	"\">\n"		\
 		  "        <string>%s</string>\n"			\
 		  "      </attribute>\n", text);			\
@@ -647,7 +649,7 @@ ochusha_bbs_thread_read_threadlist_element(OchushaBBSThread *thread,
 
 static void
 ochusha_bbs_thread_write_threadlist_element(OchushaBBSThread *thread,
-					    FILE *threadlist_xml)
+					    gzFile *threadlist_xml)
 {
   OUTPUT_THREAD_ATTRIBUTE_STRING(threadlist_xml, thread, id);
   OUTPUT_THREAD_ATTRIBUTE_STRING(threadlist_xml, thread, title);
