@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_board_2ch.c,v 1.25 2003/12/19 03:12:58 fuyu Exp $
+ * $Id: ochusha_board_2ch.c,v 1.26 2003/12/24 03:40:50 fuyu Exp $
  */
 
 #include "config.h"
@@ -49,6 +49,8 @@
 #include <time.h>
 #include <unistd.h>
 
+#include <zlib.h>
+
 
 #ifndef MAP_NOCORE
 # define MAP_NOCORE	0
@@ -67,7 +69,7 @@ static void ochusha_board_2ch_read_boardlist_element(
 						GHashTable *board_attributes);
 static void ochusha_board_2ch_write_boardlist_element(
 						OchushaBulletinBoard *board,
-						FILE *boardlist_xml);
+						gzFile *boardlist_xml);
 
 static const char *ochusha_board_2ch_get_subject_txt_encoding(
 						OchushaBulletinBoard *board);
@@ -238,12 +240,12 @@ ochusha_board_2ch_read_boardlist_element(OchushaBulletinBoard *board,
 }
 
 
-#define OUTPUT_BOARD_ATTRIBUTE_STRING(file, board, attribute)		\
+#define OUTPUT_BOARD_ATTRIBUTE_STRING(gzfile, board, attribute)		\
   do {									\
     if ((board)->attribute != NULL)					\
       {									\
         gchar *text = g_markup_escape_text((board)->attribute, -1);	\
-        fprintf(file,							\
+        gzprintf(gzfile,						\
 	  "        <attribute name=\"" #attribute	"\">\n"		\
 	  "          <string>%s</string>\n"				\
 	  "        </attribute>\n", text);				\
@@ -254,7 +256,7 @@ ochusha_board_2ch_read_boardlist_element(OchushaBulletinBoard *board,
 
 static void
 ochusha_board_2ch_write_boardlist_element(OchushaBulletinBoard *board,
-					  FILE *boardlist_xml)
+					  gzFile *boardlist_xml)
 {
   OchushaBoard2ch *board_2ch = OCHUSHA_BOARD_2CH(board);
 
@@ -323,7 +325,7 @@ ochusha_board_2ch_recover_threadlist(OchushaBulletinBoard *board,
 	       ochusha_bulletin_board_get_id(board)) >= PATH_MAX)
     return;
 
-  dat_dirname = ochusha_config_find_directory(config, path);
+  dat_dirname = ochusha_config_find_directory(config, path, NULL);
 
   if (dat_dirname == NULL)
     return;
@@ -359,7 +361,7 @@ ochusha_board_2ch_recover_threadlist(OchushaBulletinBoard *board,
 		       ochusha_bulletin_board_get_id(board),
 		       dat_filename) < PATH_MAX)
 	    {
-	      int fd = ochusha_config_open_file(config, path, O_RDONLY);
+	      int fd = ochusha_config_open_file(config, path, NULL, O_RDONLY);
 	      if (fd >= 0)
 		{
 		  OchushaAsyncBuffer *dat_buffer
