@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_board_category.c,v 1.6 2003/11/26 04:13:51 fuyu Exp $
+ * $Id: ochusha_board_category.c,v 1.7 2003/12/24 03:40:50 fuyu Exp $
  */
 
 #include "config.h"
@@ -43,6 +43,8 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include <zlib.h>
+
 
 static void ochusha_board_category_class_init(OchushaBoardCategoryClass *klass);
 static void ochusha_board_category_init(OchushaBoardCategory *category);
@@ -60,7 +62,7 @@ static void ochusha_board_category_read_boardlist_element(
 						GHashTable *cat_attributes);
 static void ochusha_board_category_write_boardlist_element(
 						OchushaBoardCategory *category,
-						FILE *boardlist_xml);
+						gzFile *boardlist_xml);
 
 
 GType
@@ -352,19 +354,23 @@ ochusha_board_category_lookup_board_by_name(OchushaBoardCategory *category,
 }
 
 
-#define OUTPUT_CATEGORY_ATTRIBUTE_BOOLEAN(file, category, attribute)	\
-  fprintf(file,								\
-	  "      <attribute name=\"" #attribute	"\">\n"			\
-	  "        <boolean val=\"%s\"/>\n"				\
-	  "      </attribute>\n",					\
-	  (category)->attribute ? "true" : "false")
+#define OUTPUT_CATEGORY_ATTRIBUTE_BOOLEAN(gzfile, category, attribute)	\
+  do									\
+    {									\
+      if ((category)->attribute)					\
+	gzprintf(gzfile,						\
+		 "      <attribute name=\"" #attribute	"\">\n"		\
+		 "        <boolean val=\"%s\"/>\n"			\
+		 "      </attribute>\n",				\
+		 (category)->attribute ? "true" : "false");		\
+    } while (0)
 
-#define OUTPUT_CATEGORY_ATTRIBUTE_STRING(file, category, attribute)	\
+#define OUTPUT_CATEGORY_ATTRIBUTE_STRING(gzfile, category, attribute)	\
   do {									\
     if ((category)->attribute)						\
       {									\
 	gchar *text = g_markup_escape_text((category)->attribute, -1);	\
-	fprintf(file,							\
+	gzprintf(gzfile,						\
 		"      <attribute name=\"" #attribute	"\">\n"		\
 		"        <string>%s</string>\n"				\
 		"      </attribute>\n", text);				\
@@ -383,7 +389,7 @@ ochusha_board_category_read_boardlist_element(OchushaBoardCategory *category,
 
 static void
 ochusha_board_category_write_boardlist_element(OchushaBoardCategory *category,
-					       FILE *boardlist_xml)
+					       gzFile *boardlist_xml)
 {
   OUTPUT_CATEGORY_ATTRIBUTE_STRING(boardlist_xml, category, name);
   OUTPUT_CATEGORY_ATTRIBUTE_BOOLEAN(boardlist_xml, category, hidden);
