@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_bulletin_board.c,v 1.20 2003/12/19 03:13:49 fuyu Exp $
+ * $Id: ochusha_bulletin_board.c,v 1.21 2003/12/24 03:40:50 fuyu Exp $
  */
 
 #include "config.h"
@@ -45,6 +45,8 @@
 #include <string.h>
 #include <unistd.h>
 
+#include <zlib.h>
+
 
 static void ochusha_bulletin_board_class_init(OchushaBulletinBoardClass *klass);
 static void ochusha_bulletin_board_init(OchushaBulletinBoard *board);
@@ -65,7 +67,7 @@ static void ochusha_bulletin_board_read_boardlist_element(
 						GHashTable *board_attributes);
 static void ochusha_bulletin_board_write_boardlist_element(
 						OchushaBulletinBoard *board,
-						FILE *boardlist_xml);
+						gzFile *boardlist_xml);
 
 GType
 ochusha_bulletin_board_get_type(void)
@@ -695,12 +697,12 @@ ochusha_bulletin_board_create_new_thread(OchushaBulletinBoard *board,
 /*
  * boardlist.xmlϢ
  */
-#define OUTPUT_BOARD_ATTRIBUTE_BOOLEAN(file, board, attribute)		\
+#define OUTPUT_BOARD_ATTRIBUTE_BOOLEAN(gzfile, board, attribute)	\
   do									\
     {									\
       if ((board)->attribute)						\
 	{								\
-	  fprintf(file,							\
+	  gzprintf(gzfile,						\
 		  "        <attribute name=\"" #attribute "\">\n"	\
 		  "          <boolean val=\"%s\"/>\n"			\
 		  "        </attribute>\n",				\
@@ -709,12 +711,12 @@ ochusha_bulletin_board_create_new_thread(OchushaBulletinBoard *board,
     } while (0)
 
 
-#define OUTPUT_BOARD_ATTRIBUTE_INT(file, board, attribute)		\
+#define OUTPUT_BOARD_ATTRIBUTE_INT(gzfile, board, attribute)		\
   do									\
     {									\
       if ((board)->attribute)						\
 	{								\
-	  fprintf(file,							\
+	  gzprintf(gzfile,						\
 		  "        <attribute name=\"" #attribute "\">\n"	\
 		  "          <int val=\"%d\"/>\n"			\
 		  "        </attribute>\n", (board)->attribute);	\
@@ -722,13 +724,13 @@ ochusha_bulletin_board_create_new_thread(OchushaBulletinBoard *board,
     } while (0)
 
 
-#define OUTPUT_BOARD_ATTRIBUTE_STRING(file, board, attribute)		\
+#define OUTPUT_BOARD_ATTRIBUTE_STRING(gzfile, board, attribute)		\
   do									\
     {									\
       if ((board)->attribute)						\
 	{								\
 	  gchar *text = g_markup_escape_text((board)->attribute, -1);	\
-	  fprintf(file,							\
+	  gzprintf(gzfile,						\
 		  "        <attribute name=\"" #attribute "\">\n"	\
 		  "          <string>%s</string>\n"			\
 		  "        </attribute>\n", text);			\
@@ -748,7 +750,7 @@ ochusha_bulletin_board_read_boardlist_element(OchushaBulletinBoard *board,
 
 static void
 ochusha_bulletin_board_write_boardlist_element(OchushaBulletinBoard *board,
-					       FILE *boardlist_xml)
+					       gzFile *boardlist_xml)
 {
   OUTPUT_BOARD_ATTRIBUTE_INT(boardlist_xml, board, bbs_type);
   OUTPUT_BOARD_ATTRIBUTE_STRING(boardlist_xml, board, name);
@@ -1121,6 +1123,7 @@ getEntityHandler(void *context, const xmlChar *name)
 gboolean
 ochusha_bulletin_board_read_threadlist_xml(OchushaBulletinBoard *board,
 					   OchushaConfig *config,
+					   const char *subdir,
 					   gboolean recover_mode)
 {
   SAXContext context =
@@ -1143,16 +1146,27 @@ ochusha_bulletin_board_read_threadlist_xml(OchushaBulletinBoard *board,
   fprintf(stderr, "id: %s\n", ochusha_bulletin_board_get_id(board));
 #endif
 
-  if (snprintf(pathname, PATH_MAX, "cache/%s%s%s/%s",
+  if (snprintf(pathname, PATH_MAX, "%s%s%s/%s",
 	       ochusha_bulletin_board_get_server(board),
 	       ochusha_bulletin_board_get_base_path(board),
 	       ochusha_bulletin_board_get_id(board),
 	       OCHUSHA_THREADLIST_XML) >= PATH_MAX)
     return FALSE;
 
-  threadlist_xml = ochusha_config_find_file(config, pathname);
+  threadlist_xml = ochusha_config_find_file(config, pathname, subdir);
   if (threadlist_xml == NULL)
-    return FALSE;
+    {
+      if (snprintf(pathname, PATH_MAX, "cache/%s%s%s/%s",
+		   ochusha_bulletin_board_get_server(board),
+		   ochusha_bulletin_board_get_base_path(board),
+		   ochusha_bulletin_board_get_id(board),
+		   OCHUSHA_THREADLIST_XML) >= PATH_MAX)
+	return FALSE;
+
+      threadlist_xml = ochusha_config_find_file(config, pathname, subdir);
+      if (threadlist_xml == NULL)
+	return FALSE;
+    }
 
   memset(&sax_handler, 0, sizeof(xmlSAXHandler));
 #if LIBXML_VERSION >= 20600
@@ -1202,7 +1216,7 @@ ochusha_bulletin_board_read_threadlist_xml(OchushaBulletinBoard *board,
 typedef struct _WriteThreadlistArgs
 {
   OchushaBulletinBoard *board;
-  FILE *threadlist_xml;
+  gzFile *threadlist_xml;
 } WriteThreadlistArgs;
 
 
@@ -1211,9 +1225,9 @@ write_bbs_thread(gpointer data, gpointer user_data)
 {
   OchushaBBSThread *thread = OCHUSHA_BBS_THREAD(data);
   WriteThreadlistArgs *args = (WriteThreadlistArgs *)user_data;
-  FILE *threadlist_xml = args->threadlist_xml;
+  gzFile *threadlist_xml = args->threadlist_xml;
 
-  fprintf(threadlist_xml, "    <thread>\n");
+  gzprintf(threadlist_xml, "    <thread>\n");
 
   g_signal_emit_by_name(G_OBJECT(thread),
 			"write_threadlist_element",
@@ -1224,31 +1238,44 @@ write_bbs_thread(gpointer data, gpointer user_data)
 		thread,
 		threadlist_xml);
 
-  fprintf(threadlist_xml, "    </thread>\n");
+  gzprintf(threadlist_xml, "    </thread>\n");
 }
 
 
 gboolean
 ochusha_bulletin_board_write_threadlist_xml(OchushaBulletinBoard *board,
-					    OchushaConfig *config)
+					    OchushaConfig *config,
+					    const char *subdir)
 {
   WriteThreadlistArgs args;
   char pathname[PATH_MAX];
   int fd;
-  FILE *threadlist_xml;
+  gzFile *threadlist_xml;
 
   g_return_val_if_fail(OCHUSHA_IS_BULLETIN_BOARD(board) && config != NULL,
 		       FALSE);
   g_return_val_if_fail(config->home != NULL, FALSE);
 
-  if (snprintf(pathname, PATH_MAX, "cache/%s%s%s/%s",
-	       ochusha_bulletin_board_get_server(board),
-	       ochusha_bulletin_board_get_base_path(board),
-	       ochusha_bulletin_board_get_id(board),
-	       OCHUSHA_THREADLIST_XML) >= PATH_MAX)
-    return FALSE;
+  if (subdir != NULL)
+    {
+      if (snprintf(pathname, PATH_MAX, "%s%s%s/%s",
+		   ochusha_bulletin_board_get_server(board),
+		   ochusha_bulletin_board_get_base_path(board),
+		   ochusha_bulletin_board_get_id(board),
+		   OCHUSHA_THREADLIST_XML) >= PATH_MAX)
+	return FALSE;
+    }
+  else
+    {
+      if (snprintf(pathname, PATH_MAX, "cache/%s%s%s/%s",
+		   ochusha_bulletin_board_get_server(board),
+		   ochusha_bulletin_board_get_base_path(board),
+		   ochusha_bulletin_board_get_id(board),
+		   OCHUSHA_THREADLIST_XML) >= PATH_MAX)
+	return FALSE;
+    }
 
-  fd = ochusha_config_open_file(config, pathname,
+  fd = ochusha_config_open_file(config, pathname, subdir,
 				O_WRONLY | O_TRUNC | O_CREAT);
   if (fd < 0)
     {
@@ -1257,7 +1284,7 @@ ochusha_bulletin_board_write_threadlist_xml(OchushaBulletinBoard *board,
       return FALSE;
     }
 
-  threadlist_xml = fdopen(fd, "w");
+  threadlist_xml = gzdopen(fd, "w");
   if (threadlist_xml == NULL)
     {
       close(fd);
@@ -1265,18 +1292,18 @@ ochusha_bulletin_board_write_threadlist_xml(OchushaBulletinBoard *board,
       return FALSE;
     }
 
-  fprintf(threadlist_xml, "<?xml version=\"1.0\"?>\n");
-  fprintf(threadlist_xml, "<ochusha>\n");
-  fprintf(threadlist_xml, "  <threadlist>\n");
+  gzprintf(threadlist_xml, "<?xml version=\"1.0\"?>\n");
+  gzprintf(threadlist_xml, "<ochusha>\n");
+  gzprintf(threadlist_xml, "  <threadlist>\n");
 
   args.board = board;
   args.threadlist_xml = threadlist_xml;
 
   g_slist_foreach(board->thread_list, write_bbs_thread, &args);
 
-  fprintf(threadlist_xml, "  </threadlist>\n");
-  fprintf(threadlist_xml, "</ochusha>\n");
+  gzprintf(threadlist_xml, "  </threadlist>\n");
+  gzprintf(threadlist_xml, "</ochusha>\n");
 
-  return fclose(threadlist_xml) == 0;
+  return gzclose(threadlist_xml) == 0;
 }
 
