@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_config.c,v 1.16 2003/12/19 21:13:47 fuyu Exp $
+ * $Id: ochusha_config.c,v 1.22 2003/12/24 06:21:32 fuyu Exp $
  */
 
 #include "config.h"
@@ -46,6 +46,8 @@
 #include <sys/stat.h>
 #include <unistd.h>
 
+#include <zlib.h>
+
 
 /*
  * ochusha_config_prepare_home
@@ -64,7 +66,8 @@ ochusha_config_prepare_home(OchushaConfig *config)
   if (ochusha_home == NULL)
     {
       char buffer[PATH_MAX];
-      char *home = getenv("HOME");
+      const char *home = g_get_home_dir();
+
       if (home == NULL)
 	goto no_home_exit;
 
@@ -98,25 +101,60 @@ ochusha_config_prepare_home(OchushaConfig *config)
  * ochushahomeǥ쥯ȥФpath̾filenameĥե뤬
  * ¸ߤ뤫ɤ򸡺¸ߤˤϥѥ̾ɽʸ֤
  * callerϥѥ̾ʸפˤʤäfree()̳餦
+ *
+ * NULLdirͿ줿硢
+ * ochusha_home/<dir>/filenameõĤä餽֤
+ * ˡochusha_home/filenameõĤä餽֤
+ * Ȥư򤹤롣
  */
 char *
-ochusha_config_find_file(const OchushaConfig *config, const char *filename)
+ochusha_config_find_file(const OchushaConfig *config, const char *filename,
+			 const char *dir)
 {
-  int stat_result;
   struct stat sb;
+  int len;
   char pathname[PATH_MAX];
 
   if (config->home == NULL)
     return NULL;
 
+  if (dir != NULL)
+    {
+      if (snprintf(pathname, PATH_MAX, "%s/%s/%s", config->home, dir, filename)
+	  < PATH_MAX)
+	{
+	  if (stat(pathname, &sb) == 0
+	      && S_ISREG(sb.st_mode) && access(pathname, R_OK) == 0)
+	    return G_STRDUP(pathname);
+
+	  len = strlen(pathname);
+	  if (len > 3 && strcmp(pathname + len - 3, ".gz") == 0)
+	    {
+	      pathname[len - 3] = '\0';
+	      if (stat(pathname, &sb) == 0
+		  && S_ISREG(sb.st_mode) && access(pathname, R_OK) == 0)
+		return G_STRDUP(pathname);
+	    }
+	}
+    }
+
   if (snprintf(pathname, PATH_MAX, "%s/%s", config->home, filename)
       >= PATH_MAX)
     return NULL;
 
-  stat_result = stat(pathname, &sb);
-  if (stat_result == 0 && S_ISREG(sb.st_mode) && access(pathname, R_OK) == 0)
+  if (stat(pathname, &sb) == 0
+      && S_ISREG(sb.st_mode) && access(pathname, R_OK) == 0)
     return G_STRDUP(pathname);
 
+  len = strlen(pathname);
+  if (len > 3 && strcmp(pathname + len - 3, ".gz") == 0)
+    {
+      pathname[len - 3] = '\0';
+      if (stat(pathname, &sb) == 0
+	  && S_ISREG(sb.st_mode) && access(pathname, R_OK) == 0)
+	return G_STRDUP(pathname);
+    }
+
   return NULL;
 }
 
@@ -127,24 +165,39 @@ ochusha_config_find_file(const OchushaConfig *config, const char *filename)
  * ochushahomeǥ쥯ȥФpath̾filenameĥǥ쥯ȥ꤬
  * ¸ߤ뤫ɤ򸡺¸ߤˤϥѥ̾ɽʸ֤
  * callerϥѥ̾ʸפˤʤäfree()̳餦
+ * NULLdirͿ줿硢
+ *
+ * ochusha_home/<dir>/filenameõĤä餽֤
+ * ˡochusha_home/filenameõĤä餽֤
+ * Ȥư򤹤롣
  */
 char *
 ochusha_config_find_directory(const OchushaConfig *config,
-			      const char *filename)
+			      const char *filename, const char *dir)
 {
-  int stat_result;
   struct stat sb;
   char pathname[PATH_MAX];
 
   if (config->home == NULL)
     return NULL;
 
+  if (dir != NULL)
+    {
+      if (snprintf(pathname, PATH_MAX, "%s/%s/%s", config->home, dir, filename)
+	  < PATH_MAX)
+	{
+	  if (stat(pathname, &sb) == 0
+	      && S_ISDIR(sb.st_mode) && access(pathname, R_OK) == 0)
+	    return G_STRDUP(pathname);
+	}
+    }
+
   if (snprintf(pathname, PATH_MAX, "%s/%s", config->home, filename)
       >= PATH_MAX)
     return NULL;
 
-  stat_result = stat(pathname, &sb);
-  if (stat_result == 0 && S_ISDIR(sb.st_mode) && access(pathname, R_OK) == 0)
+  if (stat(pathname, &sb) == 0
+      && S_ISDIR(sb.st_mode) && access(pathname, R_OK) == 0)
     return G_STRDUP(pathname);
 
   return NULL;
@@ -156,24 +209,78 @@ ochusha_config_find_directory(const OchushaConfig *config,
  *
  * ochushahomeǥ쥯ȥФpath̾filenameĥե
  * flagsͿ줿⡼ɤǳ
+ *
+ * flagsO_CREATĤƤʤϡ
+ * ochusha_home/<dir>/filenameõĤä餽򳫤
+ * ˡochusha_home/filenameõĤä餽򳫤
+ * Ȥư򤹤롣
+ *
+ * flagsO_CREATĤƤϡ
+ * ochusha_home/<dir>/filename򳫤
+ * Ȥư򤹤롣
  */
 int
 ochusha_config_open_file(const OchushaConfig *config, const char *filename,
-			 int flags)
+			 const char *dir, int flags)
 {
   int result;
-  int len;
+  int len = 0;
   char pathname[PATH_MAX];
 
   if (config->home == NULL)
     return -1;
 
-  len = snprintf(pathname, PATH_MAX, "%s/%s", config->home, filename);
-  if (len >= PATH_MAX)
-    return -1;
+  if (dir != NULL)
+    {
+      len = snprintf(pathname, PATH_MAX, "%s/%s/%s",
+		     config->home, dir, filename);
+      if (len < PATH_MAX)
+	{
+	  if ((flags & O_CREAT) == 0)
+	    {
+	      struct stat sb;
+	      if (stat(pathname, &sb) == 0
+		  && S_ISREG(sb.st_mode) && access(pathname, R_OK) == 0)
+		return open(pathname, flags);
 
-  if ((flags & O_CREAT) == 0)
-    return open(pathname, flags);
+	      if (len > 3 && strcmp(pathname + len - 3, ".gz") == 0)
+		{
+		  pathname[len - 3] = '\0';
+		  if (stat(pathname, &sb) == 0
+		      && S_ISREG(sb.st_mode) && access(pathname, R_OK) == 0)
+		    return open(pathname, flags);
+		}
+
+	      /* O_CREAT̵ξΤdirȴĴ */
+	      len = snprintf(pathname, PATH_MAX, "%s/%s", config->home,
+			     filename);
+	      if (len >= PATH_MAX)
+		return -1;
+
+	      if (stat(pathname, &sb) == 0
+		  && S_ISREG(sb.st_mode) && access(pathname, R_OK) == 0)
+		return open(pathname, flags);
+
+	      if (len > 3 && strcmp(pathname + len - 3, ".gz") == 0)
+		{
+		  pathname[len - 3] = '\0';
+		  if (stat(pathname, &sb) == 0
+		      && S_ISREG(sb.st_mode) && access(pathname, R_OK) == 0)
+		    return open(pathname, flags);
+		}
+	      return -1;
+	    }
+	}
+      else
+	len = 0;	/* path̾Ĺ */
+    }
+
+  if (len == 0)
+    {
+      len = snprintf(pathname, PATH_MAX, "%s/%s", config->home, filename);
+      if (len >= PATH_MAX)
+	return -1;
+    }
 
   result = open(pathname, flags, 0x180);
   if (result >= 0)
@@ -197,6 +304,83 @@ ochusha_config_open_file(const OchushaConfig *config, const char *filename,
 }
 
 
+void
+ochusha_config_unlink_file(const OchushaConfig *config,
+			   const char *filename, const char *dir)
+{
+  struct stat sb;
+  char pathname[PATH_MAX];
+
+  if (config->home == NULL)
+    return;
+
+  if (dir != NULL)
+    {
+      if (snprintf(pathname, PATH_MAX, "%s/%s/%s", config->home, dir, filename)
+	  >= PATH_MAX)
+	return;
+    }
+  else
+    {
+      if (snprintf(pathname, PATH_MAX, "%s/%s", config->home, filename)
+	  >= PATH_MAX)
+	return;
+    }
+
+  if (stat(pathname, &sb) == 0
+      && (S_ISREG(sb.st_mode) || S_ISLNK(sb.st_mode)))
+    unlink(pathname);
+}
+
+
+void
+ochusha_config_unlink_directory(const OchushaConfig *config,
+				const char *dirname)
+{
+  struct stat sb;
+  char pathname[PATH_MAX];
+  GDir *dir_list;
+
+  if (config->home == NULL)
+    return;
+
+  if (snprintf(pathname, PATH_MAX, "%s/%s", config->home, dirname)
+      >= PATH_MAX)
+    return;
+
+  if (stat(pathname, &sb) != 0 || !S_ISDIR(sb.st_mode))
+    return;
+
+  dir_list = g_dir_open(pathname, 0, NULL);
+  if (dir_list != NULL)
+    {
+      const char *name;
+      char tmp_path[PATH_MAX];
+
+      while ((name = g_dir_read_name(dir_list)) != NULL)
+	{
+	  if (snprintf(tmp_path, PATH_MAX, "%s/%s", pathname, name) < PATH_MAX)
+	    {
+	      if (stat(tmp_path, &sb) == 0)
+		{
+		  if (S_ISDIR(sb.st_mode))
+		    {
+		      if (snprintf(tmp_path, PATH_MAX, "%s/%s",
+				   dirname, name) < PATH_MAX)
+			ochusha_config_unlink_directory(config, tmp_path);
+		    }
+		  else
+		    ochusha_config_unlink_file(config, name, dirname);
+		}
+	    }
+	}
+      g_dir_close(dir_list);
+    }
+
+  rmdir(pathname);
+}
+
+
 /*
  * ochusha_config_cache_file_exist:
  *
@@ -252,7 +436,6 @@ ochusha_config_cache_find_file(const OchushaConfig *config, const char *url)
 {
   char *cur_pos;
   char pathname[PATH_MAX];
-  int stat_result;
   struct stat sb;
 
   if (url == NULL)
@@ -275,16 +458,16 @@ ochusha_config_cache_find_file(const OchushaConfig *config, const char *url)
       >= PATH_MAX)
     return NULL;
 
-  stat_result = stat(pathname, &sb);
-  if (stat_result == 0 && S_ISREG(sb.st_mode) && access(pathname, R_OK) == 0)
+  if (stat(pathname, &sb) == 0
+      && S_ISREG(sb.st_mode) && access(pathname, R_OK) == 0)
     return G_STRDUP(pathname);
 
   if (snprintf(pathname, PATH_MAX, "%s/cache/%s.gz", config->home, url + 7)
       >= PATH_MAX)
     return NULL;
 
-  stat_result = stat(pathname, &sb);
-  if (stat_result == 0 && S_ISREG(sb.st_mode) && access(pathname, R_OK) == 0)
+  if (stat(pathname, &sb) == 0
+      && S_ISREG(sb.st_mode) && access(pathname, R_OK) == 0)
     return G_STRDUP(pathname);
 
   return NULL;
@@ -388,7 +571,7 @@ ochusha_config_cache_unlink_file(const OchushaConfig *config, const char *url)
   if (len >= PATH_MAX)
     return;
 
-  pathname = ochusha_config_find_file(config, buffer);
+  pathname = ochusha_config_find_file(config, buffer, NULL);
   if (pathname == NULL)
     return;
 
@@ -719,24 +902,24 @@ getEntityHandler(void *context, const xmlChar *name)
 /*
  * config.xmlϢ
  */
-#define OUTPUT_CONFIG_ATTRIBUTE_INT(file, config, attribute)	\
-  fprintf(file,							\
+#define OUTPUT_CONFIG_ATTRIBUTE_INT(gzfile, config, attribute)	\
+  gzprintf(gzfile,						\
 	  "    <attribute name=\"" #attribute	"\">\n"		\
 	  "      <int val=\"%d\"/>\n"				\
 	  "    </attribute>\n", (config)->attribute)
 
-#define OUTPUT_CONFIG_ATTRIBUTE_BOOLEAN(file, config, attribute)\
-  fprintf(file,							\
-	  "    <attribute name=\"" #attribute	"\">\n"		\
-	  "      <boolean val=\"%s\"/>\n"			\
+#define OUTPUT_CONFIG_ATTRIBUTE_BOOLEAN(gzfile, config, attribute)	\
+  gzprintf(gzfile,							\
+	  "    <attribute name=\"" #attribute	"\">\n"			\
+	  "      <boolean val=\"%s\"/>\n"				\
 	  "    </attribute>\n", (config)->attribute ? "true" : "false" )
 
-#define OUTPUT_CONFIG_ATTRIBUTE_STRING(file, config, attribute)		\
+#define OUTPUT_CONFIG_ATTRIBUTE_STRING(gzfile, config, attribute)	\
   do {									\
     if ((config)->attribute != NULL)					\
       {									\
 	gchar *text = g_markup_escape_text((config)->attribute, -1);	\
-	fprintf(file,							\
+	gzprintf(gzfile,						\
 		"    <attribute name=\"" #attribute	"\">\n"		\
 		"      <string>%s</string>\n"				\
 		"    </attribute>\n", text);				\
@@ -746,40 +929,35 @@ getEntityHandler(void *context, const xmlChar *name)
 
 
 gboolean
-ochusha_write_config_xml(OchushaConfig *config,
+ochusha_write_config_xml(OchushaConfig *config, const char *subdir,
 			 WritePreferenceElementFunc *optional,
 			 gpointer user_data)
 {
-  FILE *config_xml;
-  int len;
-  char pathname[PATH_MAX];
+  int fd;
+  gzFile *config_xml;
   char *empty_string = "";
 
-  if (config->home == NULL)
+  fd = ochusha_config_open_file(config, OCHUSHA_CONFIG_XML, subdir,
+				O_WRONLY | O_TRUNC | O_CREAT);
+  if (fd < 0)
     {
-      fprintf(stderr, "No home!\n");
+      if (subdir == NULL)
+	fprintf(stderr, "Couldn't open " OCHUSHA_CONFIG_XML " to write.\n");
+      else
+	fprintf(stderr, "Couldn't open %s/" OCHUSHA_CONFIG_XML " to write.\n",
+		subdir);
       return FALSE;
     }
-
-  len = snprintf(pathname, PATH_MAX, "%s/%s", config->home,
-		 OCHUSHA_CONFIG_XML);
-
-  if (len >= PATH_MAX)
-    {
-      fprintf(stderr, "Too long path name.\n");
-      return FALSE;
-    }
-
-  config_xml = fopen(pathname, "w");
+  config_xml = gzdopen(fd, "w");
   if (config_xml == NULL)
     {
-      fprintf(stderr, "Couldn't open \"%s\" to write.\n", pathname);
+      fprintf(stderr, "Couldn't fdopen for config.xml.\n");
       return FALSE;
     }
 
-  fprintf(config_xml, "<?xml version=\"1.0\"?>\n");
-  fprintf(config_xml, "<ochusha>\n");
-  fprintf(config_xml, "  <preference>\n");
+  gzprintf(config_xml, "<?xml version=\"1.0\"?>\n");
+  gzprintf(config_xml, "<ochusha>\n");
+  gzprintf(config_xml, "  <preference>\n");
 
   OUTPUT_CONFIG_ATTRIBUTE_STRING(config_xml, config, bbsmenu_url);
   OUTPUT_CONFIG_ATTRIBUTE_BOOLEAN(config_xml, config, offline);
@@ -813,14 +991,14 @@ ochusha_write_config_xml(OchushaConfig *config,
   else
     fprintf(stderr, "No optional preference?\n");
 
-  fprintf(config_xml, "  </preference>\n");
-  fprintf(config_xml, "</ochusha>\n");
-  return fclose(config_xml) == 0;
+  gzprintf(config_xml, "  </preference>\n");
+  gzprintf(config_xml, "</ochusha>\n");
+  return gzclose(config_xml) == 0;
 }
 
 
 void
-ochusha_read_config_xml(OchushaConfig *config,
+ochusha_read_config_xml(OchushaConfig *config, const char *subdir,
 			ReadPreferenceElementFunc *optional,
 			gpointer user_data)
 {
@@ -831,7 +1009,8 @@ ochusha_read_config_xml(OchushaConfig *config,
       NULL		/* pref_attributes */
     };
   xmlSAXHandler sax_handler;
-  char *pathname = ochusha_config_find_file(config, OCHUSHA_CONFIG_XML);
+  char *pathname = ochusha_config_find_file(config, OCHUSHA_CONFIG_XML,
+					    subdir);
   if (pathname == NULL)
     {
       config->bbsmenu_url = G_STRDUP(OCHUSHA_DEFAULT_BBSMENU_URL);
