@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_network_broker.c,v 1.37 2003/12/16 19:46:44 fuyu Exp $
+ * $Id: ochusha_network_broker.c,v 1.38 2003/12/19 03:12:13 fuyu Exp $
  */
 
 #include "config.h"
@@ -49,15 +49,9 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include <sys/types.h>
-#include <sys/mman.h>
-
 #include <unistd.h>
 
-
-#ifndef MAP_NOCORE
-# define MAP_NOCORE	0
-#endif
+#include <zlib.h>
 
 
 #define DISABLE_PERSISTENT_CONNECTION		0
@@ -71,9 +65,6 @@
 #define POLLING_INTERVAL_MILLIS	100
 
 
-#define DEBUG_MMAP		0
-
-
 static void ochusha_network_broker_class_init(OchushaNetworkBrokerClass *klass);
 static void ochusha_network_broker_init(OchushaNetworkBroker *broker);
 static void ochusha_network_broker_finalize(GObject *object);
@@ -121,8 +112,10 @@ enum {
 
 static GObjectClass *parent_class = NULL;
 static gint broker_signals[LAST_SIGNAL] = { 0, 0, 0, 0, 0, 0 };
+static GQuark broker_id;
 static GQuark broker_buffer_status_id;
 static GQuark broker_job_args_id;
+static GQuark filedesc_id;
 static GQuark worker_sync_object_id;
 
 
@@ -200,6 +193,10 @@ ochusha_network_broker_class_init(OchushaNetworkBrokerClass *klass)
     = g_quark_from_static_string("OchushaNetworkBroker::JobArgs");
   worker_sync_object_id
     = g_quark_from_static_string("OchushaNetworkBroker::SyncObject");
+  broker_id
+    = g_quark_from_static_string("OchushaNetworkBroker::Broker");
+  filedesc_id
+    = g_quark_from_static_string("OchushaNetworkBroker::FileDescriptor");
 
   klass->access_started = NULL;
   klass->access_progressed = NULL;
@@ -247,17 +244,6 @@ ochusha_network_broker_output_log(OchushaNetworkBroker *broker,
 }
 
 
-static void
-munmap_when_finished(OchushaAsyncBuffer *buffer)
-{
-  munmap((void *)buffer->buffer, buffer->length);
-#if DEBUG_MMAP
-  fprintf(stderr, "mmapped buffer(at %p) is unmapped.\n",
-	  buffer->buffer);
-#endif
-}
-
-
 static void
 ochusha_network_broker_buffer_status_free(OchushaNetworkBrokerBufferStatus *status)
 {
@@ -269,89 +255,19 @@ ochusha_network_broker_buffer_status_free(OchushaNetworkBrokerBufferStatus *stat
 }
 
 
-static OchushaAsyncBuffer *
-get_mmapped_buffer(int fd)
-{
-  char *buf;
-  off_t len;
-  OchushaAsyncBuffer *buffer;
-  OchushaNetworkBrokerBufferStatus *status;
-
-  if (fd < 0)
-    return NULL;
-
-  status = G_NEW0(OchushaNetworkBrokerBufferStatus, 1);
-  status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_CACHE_HIT;
-
-  len = lseek(fd, 0, SEEK_END);
-  buf = mmap(NULL, len, PROT_READ, MAP_NOCORE | MAP_PRIVATE, fd, 0);
-  if (buf == MAP_FAILED)
-    {
-      fprintf(stderr, "mmap failed due to: %s (%d)\n", strerror(errno), errno);
-      G_FREE(status);
-      return NULL;
-    }
-  close(fd);
-
-#if DEBUG_MMAP
-  fprintf(stderr, "allocating mmapped buffer at %p\n", buf);
-#endif
-
-  buffer = ochusha_async_buffer_new(buf, len, munmap_when_finished);
-  g_object_set_qdata_full(G_OBJECT(buffer), broker_buffer_status_id, status,
-		(GDestroyNotify)ochusha_network_broker_buffer_status_free);
-  if (!ochusha_async_buffer_fix(buffer,
-			"ochusha_network_broker.c: get_mmapped_buffer()"))
-    {
-#if DEBUG_ASYNC_BUFFER_MOST
-      fprintf(stderr, "buffer has been terminated.\n");
-#endif
-      /* ȤϤɤ褦ʤ */
-    }
-#if DEBUG_NETWORK_MOST
-  fprintf(stderr, "get_mmapped_buffer(): buffer=%p\n", buffer);
-#endif
-
-  return buffer;
-}
-
-
 static int
 read_cache_to_buffer(const OchushaConfig *config, const char *url,
 		     OchushaAsyncBuffer *buffer)
 {
-  int len;
   int fd = ochusha_config_cache_open_file(config, url, O_RDONLY);
 
   if (fd < 0)
     return 0;	/* åʤξ */
 
   /* å夢ꡣ*/
-  len = lseek(fd, 0, SEEK_END);
-  lseek(fd, 0, SEEK_SET);
-  if (ochusha_async_buffer_resize(buffer, len,
-			"ochusha_network_broker.c: read_cache_to_buffer()"))
-    {
-      len = read(fd, (void *)buffer->buffer, len);
-      if (!ochusha_async_buffer_update_length(buffer, len,
-			"ochusha_network_broker.c: read_cache_to_buffer()"))
-	{
-#if DEBUG_ASYNC_BUFFER
-	  fprintf(stderr, "read_cache_to_buffer(): buffer may have been terminated.\n");
-#endif
-	  /* ξϤɤ褦ʤ */
-	}
-    }
-  else
-    {
-#if DEBUG_ASYNC_BUFFER
-      fprintf(stderr, "read_cache_to_buffer(): buffer may have been terminated.\n");
-#endif
-      len = 0;	/* Out of memory? */
-    }
-  close(fd);
+  ochusha_async_buffer_read_file(buffer, fd);
 
-  return len;
+  return buffer->length;
 }
 
 
@@ -359,24 +275,55 @@ static void
 write_buffer_to_cache(OchushaNetworkBroker *broker, const char *url,
 		      OchushaAsyncBuffer *buffer)
 {
+  char *tmp_pos;
   gchar message[LOG_MESSAGE_SIZE];
   int fd = ochusha_config_cache_open_file(broker->config, url,
 					  O_WRONLY | O_TRUNC | O_CREAT);
   if (fd >= 0)
     {
-      ssize_t len = write(fd, (void *)buffer->buffer, buffer->length);
-      close(fd);
-      if (len != buffer->length)
+      if ((tmp_pos = strstr(url, ".dat.gz")) != NULL && tmp_pos[7] == '\0')
 	{
-	  ochusha_config_cache_unlink_file(broker->config, url);
-	  snprintf(message, LOG_MESSAGE_SIZE,
-		   _("Couldn't update cache file for %s: %s (%d)\n"),
-		   url, strerror(errno), errno);
-	  ochusha_network_broker_output_log(broker, message);
+	  /* Ķad-hoc塼fordat.gz */
+	  int len = 0;
+	  gzFile gzfile = gzdopen(fd, "wb");
+
+	  if (gzfile != NULL)
+	    {
+	      len = gzwrite(gzfile, (void *)buffer->buffer, buffer->length);
+	      gzclose(gzfile);
+	    }
+	  else
+	    close(fd);
+
+	  if (len == 0)
+	    {
+	      ochusha_config_cache_unlink_file(broker->config, url);
+	      snprintf(message, LOG_MESSAGE_SIZE,
+		       _("Couldn't update cache file for %s: %s (%d)\n"),
+		       url, strerror(errno), errno);
+	      ochusha_network_broker_output_log(broker, message);
 #if DEBUG_NETWORK_MOST
-	  fprintf(stderr, "Couldn't update cache file for %s: %s (%d)\n",
-		  url, strerror(errno), errno);
+	      fprintf(stderr, "Couldn't update cache file for %s: %s (%d)\n",
+		      url, strerror(errno), errno);
 #endif
+	    }
+	}
+      else
+	{
+	  ssize_t len = write(fd, (void *)buffer->buffer, buffer->length);
+	  close(fd);
+	  if (len != buffer->length)
+	    {
+	      ochusha_config_cache_unlink_file(broker->config, url);
+	      snprintf(message, LOG_MESSAGE_SIZE,
+		       _("Couldn't update cache file for %s: %s (%d)\n"),
+		       url, strerror(errno), errno);
+	      ochusha_network_broker_output_log(broker, message);
+#if DEBUG_NETWORK_MOST
+	      fprintf(stderr, "Couldn't update cache file for %s: %s (%d)\n",
+		      url, strerror(errno), errno);
+#endif
+	    }
 	}
     }
   else
@@ -755,6 +702,14 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
   ghttp_status state = ghttp_not_done;
   ghttp_current_status current_status;
 
+  const char *tmp_pos;
+  gboolean force_gunzip;	/* Ķad-hock塼fordat.gz */
+
+  if ((tmp_pos = strstr(url, ".dat.gz")) != NULL && tmp_pos[7] == '\0')
+    force_gunzip = TRUE;
+  else
+    force_gunzip = FALSE;
+
   request = ghttp_request_new();
   args->request = request;	/* 塹Τˡġ */
 
@@ -972,6 +927,16 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
 		  goto error_exit;
 
 		case IDENTITY_ENCODING:
+		  if (force_gunzip)
+		    {
+		      gzip_buffer = gzip_buffer_new(buffer);
+		      if (gzip_buffer == NULL)
+			{
+			  error_message = _("Out of memory.");
+			  goto error_exit;
+			}
+		      content_encoding = GZIP_ENCODING;
+		    }
 		  break;
 		}
 	    }
@@ -1007,8 +972,7 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
 	    }
 	  else
 	    {
-	      if (!ochusha_async_buffer_append_data(buffer, body, body_len,
-			"ochusha_network_broker.c: http_read_from_url"))
+	      if (!ochusha_async_buffer_append_data(buffer, body, body_len))
 		{	/* content_encoding == IDENTITY_ENCODING */
 		  g_signal_emit(G_OBJECT(broker),
 				broker_signals[ACCESS_TERMINATED_SIGNAL],
@@ -1036,8 +1000,7 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
 	}
       else
 	{
-	  if (!ochusha_async_buffer_signal(buffer,
-			"ochusha_network_broker.c: http_read_from_url"))
+	  if (!ochusha_async_buffer_signal(buffer))
 	    {
 	      g_signal_emit(G_OBJECT(broker),
 			    broker_signals[ACCESS_TERMINATED_SIGNAL],
@@ -1355,8 +1318,7 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
   const char *error_message = NULL;
   gchar message[LOG_MESSAGE_SIZE];
 
-  if (!ochusha_async_buffer_active_ref(buffer,
-			"ochusha_network_broker.c: try_update_cache()"))
+  if (!ochusha_async_buffer_active_ref(buffer))
     {
       g_signal_emit(G_OBJECT(broker),
 		    broker_signals[ACCESS_TERMINATED_SIGNAL],
@@ -1373,8 +1335,7 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 		    _("Access terminated."),
 		    &signal_result);
 
-      ochusha_async_buffer_fix(buffer,
-			       "ochusha_network_broker.c: try_update_cache()");
+      ochusha_async_buffer_fix(buffer);
       g_object_set_qdata(G_OBJECT(buffer), broker_job_args_id, NULL);
       g_object_unref(G_OBJECT(buffer));
 
@@ -1479,8 +1440,7 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
       memcpy(cache_compare_buffer, (char *)buffer->buffer + offset,
 	     CACHE_COMPARE_SIZE);
 
-      if (!ochusha_async_buffer_update_length(buffer, offset,
-			"ochusha_network_broker.c: try_update_cache()"))
+      if (!ochusha_async_buffer_update_length(buffer, offset))
 	{
 	  g_signal_emit(G_OBJECT(broker),
 			broker_signals[ACCESS_TERMINATED_SIGNAL],
@@ -1550,8 +1510,7 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 		  else
 		    status->date = NULL;
 
-		  ochusha_async_buffer_update_length(buffer, len,
-			"ochusha_network_broker.c: try_update_cache()");
+		  ochusha_async_buffer_update_length(buffer, len);
 
 		  status->status_code = status_code;
 		  status->state
@@ -1676,8 +1635,7 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 		    }
 		}
 	      else if (!ochusha_async_buffer_append_data(buffer, body,
-							 body_len,
-			      "ochusha_network_broker.c: try_update_cache()"))
+							 body_len))
 		{	/* content_encoding == IDENTITY_ENCODING */
 		  g_signal_emit(G_OBJECT(broker),
 				broker_signals[ACCESS_TERMINATED_SIGNAL],
@@ -1726,8 +1684,7 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 	    }
 	  else
 	    {
-	      if (!ochusha_async_buffer_signal(buffer,
-			"ochusha_network_broker.c: try_update_cache()"))
+	      if (!ochusha_async_buffer_signal(buffer))
 		{
 		  g_signal_emit(G_OBJECT(broker),
 				broker_signals[ACCESS_TERMINATED_SIGNAL],
@@ -1820,8 +1777,7 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 	       _("Cache is dirty: %s\n"), url);
       ochusha_network_broker_output_log(broker, message);
 
-      if (!ochusha_async_buffer_update_length(buffer, 0,
-			"ochusha_network_broker.c: try_update_cache()"))
+      if (!ochusha_async_buffer_update_length(buffer, 0))
 	{
 	  g_signal_emit(G_OBJECT(broker),
 			broker_signals[ACCESS_TERMINATED_SIGNAL],
@@ -1860,10 +1816,8 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 
       write_buffer_to_cache(broker, url, buffer);
     }
-  ochusha_async_buffer_fix(buffer,
-			   "ochusha_network_broker.c: try_update_cache");
-  ochusha_async_buffer_active_unref(buffer,
-				"ochusha_network_broker.c: try_update_cache");
+  ochusha_async_buffer_fix(buffer);
+  ochusha_async_buffer_active_unref(buffer);
   g_object_set_qdata(G_OBJECT(buffer), broker_job_args_id, NULL);
   g_object_unref(G_OBJECT(buffer));
   return;
@@ -1940,10 +1894,8 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
   sync_object->finished = TRUE;
   UNLOCK_WORKER_SYNC_OBJECT(sync_object);
 
-  ochusha_async_buffer_fix(buffer,
-			   "ochusha_network_broker.c: try_update_cache");
-  ochusha_async_buffer_active_unref(buffer,
-				"ochusha_network_broker.c: try_update_cache");
+  ochusha_async_buffer_fix(buffer);
+  ochusha_async_buffer_active_unref(buffer);
   g_object_set_qdata(G_OBJECT(buffer), broker_job_args_id, NULL);
   g_object_unref(G_OBJECT(buffer));
 }
@@ -1964,8 +1916,9 @@ refresh_cache_after_read(WorkerThread *employee, OchushaAsyncBuffer *buffer)
   fprintf(stderr, "refresh_cache_after_read() thread is invoked.\n");
 #endif
 
-  if (!ochusha_async_buffer_active_ref(buffer,
-		"ochusha_network_broker.c: refresh_cache_after_read()"))
+  
+
+  if (!ochusha_async_buffer_active_ref(buffer))
     {
       status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_ACCESS_FAILED;
       g_signal_emit(G_OBJECT(broker),
@@ -2054,12 +2007,10 @@ refresh_cache_after_read(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 	}
     }
 
-  ochusha_async_buffer_active_unref(buffer,
-		"ochusha_network_broker.c: refresh_cache_after_read()");
+  ochusha_async_buffer_active_unref(buffer);
 
  finish_refresh_cache_after_read:
-  ochusha_async_buffer_fix(buffer,
-		"ochusha_network_broker.c: refresh_cache_after_read()");
+  ochusha_async_buffer_fix(buffer);
   g_object_set_qdata(G_OBJECT(buffer), broker_job_args_id, NULL);
   g_object_unref(G_OBJECT(buffer));
 }
@@ -2079,8 +2030,7 @@ force_read(WorkerThread *employee, OchushaAsyncBuffer *buffer)
   fprintf(stderr, "force_read() thread is invoked.\n");
 #endif
 
-  if (!ochusha_async_buffer_active_ref(buffer,
-				"ochusha_network_broker.c: force_read()"))
+  if (!ochusha_async_buffer_active_ref(buffer))
     {
       status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_ACCESS_FAILED;
       g_signal_emit(G_OBJECT(broker),
@@ -2116,16 +2066,65 @@ force_read(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 #endif
     }
 
-  ochusha_async_buffer_active_unref(buffer,
-				    "ochusha_network_broker.c: force_read()");
+  ochusha_async_buffer_active_unref(buffer);
 
  finish_force_read:
-  ochusha_async_buffer_fix(buffer, "ochusha_network_broker.c: force_read()");
+  ochusha_async_buffer_fix(buffer);
   g_object_set_qdata(G_OBJECT(buffer), broker_job_args_id, NULL);
   g_object_unref(G_OBJECT(buffer));
 }
 
 
+static void
+background_read_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
+{
+  gboolean signal_result;
+  int fd = (int)g_object_get_qdata(G_OBJECT(buffer), filedesc_id);
+  OchushaNetworkBroker *broker = g_object_get_qdata(G_OBJECT(buffer),
+						    broker_id);
+  OchushaNetworkBrokerBufferStatus *status
+    = g_object_get_qdata(G_OBJECT(buffer), broker_buffer_status_id);
+
+  if (!ochusha_async_buffer_active_ref(buffer))
+    {
+      status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_ACCESS_FAILED;
+      g_signal_emit(G_OBJECT(broker),
+		    broker_signals[ACCESS_TERMINATED_SIGNAL],
+		    0,
+		    buffer,
+		    &signal_result);
+      goto finish_job;
+    }
+
+  status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_CACHE_HIT;
+
+  if (ochusha_async_buffer_read_file(buffer, fd))
+    {
+      g_signal_emit(G_OBJECT(broker),
+		    broker_signals[ACCESS_COMPLETED_SIGNAL],
+		    0,
+		    buffer,
+		    &signal_result);
+    }
+  else
+    {
+      status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_ACCESS_FAILED;
+      g_signal_emit(G_OBJECT(broker),
+		    broker_signals[ACCESS_TERMINATED_SIGNAL],
+		    0,
+		    buffer,
+		    &signal_result);
+    }
+
+  ochusha_async_buffer_active_unref(buffer);
+
+ finish_job:
+  ochusha_async_buffer_fix(buffer);
+  g_object_unref(G_OBJECT(buffer));
+  g_object_unref(G_OBJECT(broker));
+}
+
+
 /*
  * ochusha_network_broker_read_from_url
  *
@@ -2188,22 +2187,28 @@ ochusha_network_broker_read_from_url(OchushaNetworkBroker *broker,
 					      url, O_RDONLY);
       if (fd >= 0)
 	{
-	  OchushaAsyncBuffer *buffer = get_mmapped_buffer(fd);
+	  WorkerJob *job = G_NEW0(WorkerJob, 1);
+	  OchushaAsyncBuffer *buffer = ochusha_async_buffer_new(NULL, 0, NULL);
+	  OchushaNetworkBrokerBufferStatus *status
+	    = G_NEW0(OchushaNetworkBrokerBufferStatus, 1);
+
+	  g_object_set_qdata(G_OBJECT(buffer), filedesc_id, (gpointer)fd);
+	  g_object_set_qdata(G_OBJECT(buffer), broker_id, broker);
+	  g_object_set_qdata_full(G_OBJECT(buffer), broker_buffer_status_id,
+		status,
+		(GDestroyNotify)ochusha_network_broker_buffer_status_free);
+
+	  status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_UNKNOWN;
+
+	  job->canceled = FALSE;
+	  job->job = (JobFunc *)background_read_cache;
+	  job->args = buffer;
+
+	  g_object_ref(G_OBJECT(broker));
+	  g_object_ref(G_OBJECT(buffer));
+
+	  commit_job(job);
 
-	  if (buffer != NULL)
-	    g_signal_emit(G_OBJECT(broker),
-			  broker_signals[ACCESS_COMPLETED_SIGNAL],
-			  0,
-			  buffer,
-			  &signal_result);
-	  else
-	    g_signal_emit(G_OBJECT(broker),
-			  broker_signals[ACCESS_FAILED_SIGNAL],
-			  0,
-			  NULL,
-			  OCHUSHA_NETWORK_BROKER_FAILURE_REASON_MMAP_FAILED,
-			  _("Couldn't get mmapped buffer."),
-			  &signal_result);
 	  return buffer;
 	}
 
@@ -2412,8 +2417,7 @@ http_read_from_url_synchronously(OchushaNetworkBroker *broker,
 #endif
 	    }
 	}
-      else if (!ochusha_async_buffer_append_data(buffer, body, body_len,
-		"ochusha_network_broker.c: http_read_from_url_synchronously"))
+      else if (!ochusha_async_buffer_append_data(buffer, body, body_len))
 	{	/* content_encoding == IDENTITY_ENCODING */
 	  g_signal_emit(G_OBJECT(broker),
 			broker_signals[ACCESS_TERMINATED_SIGNAL],
@@ -2510,8 +2514,7 @@ force_read_synchronously(WorkerThread *employee, OchushaAsyncBuffer *buffer)
   fprintf(stderr, "force_read_synchronously() thread is invoked.\n");
 #endif
 
-  if (!ochusha_async_buffer_active_ref(buffer,
-		"ochusha_network_broker.c: force_read_synchronously()"))
+  if (!ochusha_async_buffer_active_ref(buffer))
     {
       status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_ACCESS_FAILED;
       g_signal_emit(G_OBJECT(broker),
@@ -2547,12 +2550,10 @@ force_read_synchronously(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 #endif
     }
 
-  ochusha_async_buffer_active_unref(buffer,
-		"ochusha_network_broker.c: force_read_synchronously()");
+  ochusha_async_buffer_active_unref(buffer);
 
  finish_force_read:
-  ochusha_async_buffer_fix(buffer,
-		"ochusha_network_broker.c: force_read_synchronously()");
+  ochusha_async_buffer_fix(buffer);
   g_object_set_qdata(G_OBJECT(buffer), broker_job_args_id, NULL);
   g_object_unref(G_OBJECT(buffer));
 }
