@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_thread_2ch.c,v 1.47 2003/12/21 18:36:13 fuyu Exp $
+ * $Id: ochusha_thread_2ch.c,v 1.48 2003/12/24 03:40:50 fuyu Exp $
  */
 
 #include "config.h"
@@ -43,6 +43,8 @@
 #include <time.h>
 #include <unistd.h>
 
+#include <zlib.h>
+
 
 #define DEBUG_DAT	0
 #define DEBUG_DAT_MOST	0
@@ -60,7 +62,7 @@ static void ochusha_thread_2ch_read_threadlist_element(
 						GHashTable *thread_attributes);
 static void ochusha_thread_2ch_write_threadlist_element(
 						OchushaBBSThread *thread,
-						FILE *threadlist_xml);
+						gzFile *threadlist_xml);
 
 static void ochusha_thread_2ch_property_change_notify(OchushaThread2ch *thread);
 
@@ -275,12 +277,12 @@ ochusha_thread_2ch_read_threadlist_element(OchushaBBSThread *thread,
 }
 
 
-#define OUTPUT_THREAD_ATTRIBUTE_STRING(file, thread, attribute)		\
+#define OUTPUT_THREAD_ATTRIBUTE_STRING(gzfile, thread, attribute)	\
   do {									\
     if ((thread)->attribute != NULL)					\
       {									\
         gchar *text = g_markup_escape_text((thread)->attribute, -1);	\
-        fprintf(file,							\
+        gzprintf(gzfile,						\
 	  "      <attribute name=\"" #attribute	"\">\n"			\
 	  "        <string>%s</string>\n"				\
 	  "      </attribute>\n", text);				\
@@ -291,7 +293,7 @@ ochusha_thread_2ch_read_threadlist_element(OchushaBBSThread *thread,
 
 static void
 ochusha_thread_2ch_write_threadlist_element(OchushaBBSThread *thread,
-					    FILE *threadlist_xml)
+					    gzFile *threadlist_xml)
 {
   OchushaThread2ch *thread_2ch = OCHUSHA_THREAD_2CH(thread);
 
